@@ -1,4 +1,5 @@
 from .catalogue import Catalogue
+from .evaluator import Evaluator, OutputFormat
 from .exceptions import CatalogueError, JSONSchemaError, JSONPointerError, URIError
 from .json import JSON
 from .jsonpointer import JSONPointer
@@ -8,11 +9,15 @@ from .uri import URI
 __all__ = [
     'Catalogue',
     'CatalogueError',
+    'Evaluator',
     'JSON',
     'JSONPointer',
     'JSONPointerError',
     'JSONSchema',
     'JSONSchemaError',
+    'OutputFormat',
     'URI',
     'URIError',
 ]
+
+__version__ = '0.4.0'
