@@ -3,7 +3,20 @@ from __future__ import annotations
 from collections import deque
 from contextlib import contextmanager
 from dataclasses import dataclass
-from typing import Any, Mapping, Union, Optional, Dict, List, Iterator, Tuple, Type, Sequence, ContextManager
+from typing import (
+    Any,
+    Mapping,
+    Union,
+    Optional,
+    Dict,
+    List,
+    Iterator,
+    Tuple,
+    Type,
+    Sequence,
+    ContextManager,
+    TYPE_CHECKING,
+)
 from uuid import uuid4
 
 from jschon.exceptions import JSONSchemaError
@@ -12,6 +25,9 @@ from jschon.jsonpointer import JSONPointer
 from jschon.uri import URI
 from jschon.utils import tuplify
 
+if TYPE_CHECKING:
+    from jschon.catalogue import Catalogue
+
 __all__ = [
     'JSONSchema',
     'Metaschema',
@@ -28,34 +44,88 @@ __all__ = [
 
 
 class JSONSchema(JSON):
+    """JSON schema document model."""
 
     @staticmethod
     def iscompatible(value: Any) -> bool:
-        return isinstance(value, bool) or \
-               isinstance(value, Mapping) and all(isinstance(k, str) for k in value)
+        """Return :const:`True` if `value` can be used to initialize
+        a :class:`JSONSchema` instance, :const:`False` otherwise.
+        
+        `value` is considered schema-compatible if it is either a
+        :class:`bool` or a :class:`Mapping[str, Any]`.
+        """
+        return (isinstance(value, bool) or
+                isinstance(value, Mapping) and all(isinstance(k, str) for k in value))
 
     def __init__(
             self,
             value: Union[bool, Mapping[str, AnyJSONCompatible]],
             *,
+            catalogue: Catalogue = None,
             uri: URI = None,
             metaschema_uri: URI = None,
             parent: JSON = None,
             key: str = None,
     ):
+        """Initialize a :class:`JSONSchema` instance from the given
+        schema-compatible `value`.
+
+        :param value: a schema-compatible Python object
+        :param catalogue: the catalogue in which the schema will find its
+            metaschema, and in which it will be cached; if not given, the
+            default :class:`~jschon.catalogue.Catalogue` instance (created
+            with :meth:`~jschon.catalogue.Catalogue.create_default_catalogue`)
+            is used
+        :param uri: the URI identifying the schema; an ``"$id"`` keyword
+            appearing in `value` will override this
+        :param metaschema_uri: the URI identifying the schema's metaschema;
+            a ``"$schema"`` keyword appearing in `value` will override this
+        :param parent: the parent node of the schema; used internally when
+            creating a subschema
+        :param key: the index of the schema within its parent; used internally
+            when creating a subschema
+        """
         from jschon.catalogue import Catalogue
+
+        if catalogue is None:
+            if (catalogue := Catalogue.get_default_catalogue()) is None:
+                raise JSONSchemaError("catalogue not given and default catalogue not found")
+
         if uri is not None:
-            Catalogue.add_schema(uri, self)
+            catalogue.add_schema(uri, self)
+
+        self.catalogue: Catalogue = catalogue
+        """The catalogue provides the schema with its keyword vocabulary (via
+        the metaschema), and a (sub)schema cache."""
 
         self._uri: Optional[URI] = uri
         self._metaschema_uri: Optional[URI] = metaschema_uri
-        self.keywords: Dict[str, Keyword] = {}
 
-        # don't call super().__init__
-        self.value: Union[bool, Mapping[str, AnyJSONCompatible]]
+        self.keywords: Dict[str, Keyword] = {}
+        """A dictionary of the schema's keyword objects, indexed by keyword name."""
+
+        # do not call super().__init__
+        # all inherited attributes are initialized here:
+
+        self.value: Union[bool, Mapping[str, JSON]]
+        """The schema data.
+        
+        =========   ===============
+        JSON type   value type
+        =========   ===============
+        boolean     bool
+        object      dict[str, JSON]
+        =========   ===============
+        """
+
         self.type: str
+        """The JSON type of the schema. One of ``"boolean"``, ``"object"``."""
+
         self.parent: Optional[JSON] = parent
+        """The containing JSON or JSONSchema node."""
+
         self.key: Optional[str] = key
+        """The index of the schema within its parent."""
 
         if isinstance(value, bool):
             self.type = "boolean"
@@ -66,7 +136,7 @@ class JSONSchema(JSON):
             self.value = {}
 
             if self.parent is None and self.uri is None:
-                self.uri = URI(f'mem:{uuid4()}')
+                self.uri = URI(f'urn:uuid:{uuid4()}')
 
             self._bootstrap(value)
 
@@ -134,17 +204,28 @@ class JSONSchema(JSON):
                     break
 
     def validate(self) -> JSONSchema:
-        if not (scope := self.metaschema.evaluate(JSON(self.value))).valid:
-            messages = ''
-            for error in scope.collect_errors():
-                messages += f"\ninstance path={error.instance_path};" \
-                            f" evaluation path={error.evaluation_path};" \
-                            f" error={error.message=}"
-            raise JSONSchemaError(f"The schema is invalid against its metaschema:{messages}")
+        """Validate the schema against its metaschema.
         
+        This method returns `self`, so that schema instantiation and validation
+        may be chained.
+
+        :raise JSONSchemaError: if the schema is invalid against its metaschema
+        """
+        if not self.metaschema.evaluate(JSON(self.value)).valid:
+            raise JSONSchemaError(f"The schema is invalid against its metaschema")
+
         return self
 
     def evaluate(self, instance: JSON, scope: Scope = None) -> Scope:
+        """Evaluate a JSON document.
+        
+        The returned :class:`Scope` represents the complete evaluation
+        result tree for this (sub)schema node.
+
+        :param instance: the JSON document to evaluate
+        :param scope: the dynamic evaluation scope; used by keywords
+            when invoking this method recursively
+        """
         if scope is None:
             scope = Scope(self)
 
@@ -157,16 +238,23 @@ class JSONSchema(JSON):
         else:
             for key, keyword in self.keywords.items():
                 if keyword.can_evaluate(instance):
-                    with scope(key, self) as subscope:
+                    with scope(instance, key, self) as subscope:
                         keyword.evaluate(instance, subscope)
 
-            if any(child._assert and not child.valid for child in scope.children.values()):
+            if any(
+                    not child.valid
+                    for child in scope.iter_children(instance)
+            ):
                 scope.fail(instance, "The instance failed validation against the schema")
 
         return scope
 
     @property
     def parentschema(self) -> Optional[JSONSchema]:
+        """The containing :class:`JSONSchema` instance.
+        
+        Note that this is not necessarily the same as `self.parent`.
+        """
         parent = self.parent
         while parent is not None:
             if isinstance(parent, JSONSchema):
@@ -175,18 +263,22 @@ class JSONSchema(JSON):
 
     @property
     def metaschema(self) -> Metaschema:
-        from jschon.catalogue import Catalogue
-
+        """The schema's :class:`Metaschema`."""
         if (uri := self.metaschema_uri) is None:
             raise JSONSchemaError("The schema's metaschema URI has not been set")
 
-        if not isinstance(metaschema := Catalogue.get_schema(uri), Metaschema):
+        if not isinstance(metaschema := self.catalogue.get_schema(uri), Metaschema):
             raise JSONSchemaError(f"The schema referenced by {uri} is not a metachema")
 
         return metaschema
 
     @property
     def metaschema_uri(self) -> Optional[URI]:
+        """The :class:`~jschon.uri.URI` identifying the schema's metaschema.
+        
+        If not defined on this (sub)schema, the metaschema URI
+        is determined by the parent schema.
+        """
         if self._metaschema_uri is not None:
             return self._metaschema_uri
         if self.parentschema is not None:
@@ -198,6 +290,11 @@ class JSONSchema(JSON):
 
     @property
     def base_uri(self) -> Optional[URI]:
+        """The schema's base :class:`~jschon.uri.URI`.
+        
+        The base URI is obtained by searching up the schema tree
+        for a schema URI, and removing any fragment.
+        """
         if self._uri is not None:
             return self._uri.copy(fragment=False)
         if self.parentschema is not None:
@@ -205,23 +302,30 @@ class JSONSchema(JSON):
 
     @property
     def uri(self) -> Optional[URI]:
+        """The :class:`~jschon.uri.URI` identifying the schema.
+
+        Used as the key for caching the schema in the catalogue.
+        """
         return self._uri
 
     @uri.setter
     def uri(self, value: Optional[URI]) -> None:
-        from jschon.catalogue import Catalogue
-
         if self._uri != value:
             if self._uri is not None:
-                Catalogue.del_schema(self._uri)
+                self.catalogue.del_schema(self._uri)
 
             self._uri = value
 
             if self._uri is not None:
-                Catalogue.add_schema(self._uri, self)
+                self.catalogue.add_schema(self._uri, self)
 
     @property
     def canonical_uri(self) -> Optional[URI]:
+        """The absolute location of the (sub)schema.
+        
+        This is not necessarily an 'absolute URI', as it may contain
+        a fragment.
+        """
         if self._uri is not None:
             return self._uri
 
@@ -244,6 +348,7 @@ class Metaschema(JSONSchema):
 
     def __init__(
             self,
+            catalogue: Catalogue,
             value: Mapping[str, AnyJSONCompatible],
             core_vocabulary: Vocabulary,
             *default_vocabularies: Vocabulary,
@@ -251,7 +356,7 @@ class Metaschema(JSONSchema):
         self.core_vocabulary: Vocabulary = core_vocabulary
         self.default_vocabularies: Tuple[Vocabulary, ...] = default_vocabularies
         self.kwclasses: Dict[str, KeywordClass] = {}
-        super().__init__(value)
+        super().__init__(value, catalogue=catalogue)
 
     def _bootstrap(self, value: Mapping[str, AnyJSONCompatible]) -> None:
         super()._bootstrap(value)
@@ -313,7 +418,7 @@ class Applicator:
     @staticmethod
     def jsonify(parentschema: JSONSchema, key: str, value: AnyJSONCompatible) -> Optional[JSONSchema]:
         if JSONSchema.iscompatible(value):
-            return JSONSchema(value, parent=parentschema, key=key)
+            return JSONSchema(value, parent=parentschema, key=key, catalogue=parentschema.catalogue)
 
 
 class ArrayApplicator:
@@ -322,7 +427,7 @@ class ArrayApplicator:
     @staticmethod
     def jsonify(parentschema: JSONSchema, key: str, value: AnyJSONCompatible) -> Optional[JSON]:
         if isinstance(value, Sequence) and all(JSONSchema.iscompatible(v) for v in value):
-            return JSON(value, parent=parentschema, key=key, itemclass=JSONSchema)
+            return JSON(value, parent=parentschema, key=key, itemclass=JSONSchema, catalogue=parentschema.catalogue)
 
 
 class PropertyApplicator:
@@ -334,7 +439,7 @@ class PropertyApplicator:
                 isinstance(k, str) and JSONSchema.iscompatible(v)
                 for k, v in value.items()
         ):
-            return JSON(value, parent=parentschema, key=key, itemclass=JSONSchema)
+            return JSON(value, parent=parentschema, key=key, itemclass=JSONSchema, catalogue=parentschema.catalogue)
 
 
 @dataclass
@@ -366,17 +471,17 @@ class Scope:
         self.path: JSONPointer = path or JSONPointer()
         self.relpath: JSONPointer = relpath or JSONPointer()
         self.parent: Optional[Scope] = parent
-        self.children: Dict[str, Scope] = {}
+        self.children: Dict[JSONPointer, Dict[str, Scope]] = {}
         self.annotations: Dict[str, Annotation] = {}
         self.errors: List[Error] = []
         self._assert = True
         self._discard = False
 
     @contextmanager
-    def __call__(self, key: str, schema: JSONSchema = None) -> ContextManager[Scope]:
-        """Yield a subscope of the current scope by descending down the
-        evaluation path by ``key``, into ``schema`` if given, or within
-        the schema of the current scope otherwise."""
+    def __call__(self, instance: JSON, key: str, schema: JSONSchema = None) -> ContextManager[Scope]:
+        """Yield a subscope of the current scope, for evaluating `instance`.
+        Descend down the evaluation path by `key`, into `schema` if given, or
+        within the schema of the current scope otherwise."""
         path = self.path / key
         schema = schema or self.schema
         if schema is self.schema:
@@ -384,27 +489,25 @@ class Scope:
         else:
             relpath = JSONPointer((key,))
 
-        self.children[key] = (child := Scope(
+        self.children.setdefault(instance_path := instance.path, {})
+        self.children[instance_path][key] = (child := Scope(
             schema,
             path=path,
             relpath=relpath,
             parent=self,
         ))
+
         try:
             yield child
         finally:
             if child._discard:
-                del self.children[key]
+                del self.children[instance_path][key]
 
-    @property
-    def root(self) -> Scope:
-        scope = self
-        while scope.parent is not None:
-            scope = scope.parent
-        return scope
-
-    def sibling(self, key: str) -> Optional[Scope]:
-        return self.parent.children.get(key) if self.parent else None
+    def sibling(self, instance: JSON, key: str) -> Optional[Scope]:
+        try:
+            return self.parent.children[instance.path][key] if self.parent else None
+        except KeyError:
+            return None
 
     def annotate(self, instance: JSON, key: str, value: AnyJSONCompatible) -> None:
         if value is not None:
@@ -424,8 +527,8 @@ class Scope:
         )]
 
     def noassert(self) -> None:
-        """Indicate that the scope's validity should not affect its
-        assertion result."""
+        """Indicate that the scope should be taken to be valid,
+        regardless of errors."""
         self._assert = False
 
     def discard(self) -> None:
@@ -434,7 +537,7 @@ class Scope:
 
     @property
     def valid(self) -> bool:
-        return not self.errors
+        return not self._assert or not self.errors
 
     @property
     def absolute_uri(self) -> Optional[URI]:
@@ -445,21 +548,29 @@ class Scope:
                 relpath = self.relpath
             return schema_uri.copy(fragment=relpath.uri_fragment())
 
-    def collect_annotations(self, instance: JSON, key: str = None) -> Iterator[Annotation]:
-        """Return an iterator over annotations produced in this subtree
-        for the given instance, optionally filtered by keyword."""
-        if self.valid:
+    def iter_children(self, instance: JSON = None) -> Iterator[Scope]:
+        """Return a flattened view of child scopes, optionally filtered
+        by the instance to which they apply."""
+        for instance_path, keyword_scopes in self.children.items():
+            if instance is None or instance.path == instance_path:
+                yield from keyword_scopes.values()
+
+    def collect_annotations(self, instance: JSON = None, key: str = None) -> Iterator[Annotation]:
+        """Return an iterator over annotations produced in this subtree,
+        optionally filtered by instance and/or keyword."""
+        if not self._discard and not self.errors:
             for annotation_key, annotation in self.annotations.items():
-                if (key is None or key == annotation_key) and annotation.instance_path == instance.path:
+                if (key is None or key == annotation_key) and \
+                        (instance is None or instance.path == annotation.instance_path):
                     yield annotation
-            for child in self.children.values():
+            for child in self.iter_children():
                 yield from child.collect_annotations(instance, key)
 
     def collect_errors(self) -> Iterator[Error]:
         """Return an iterator over errors produced in this subtree."""
-        if not self.valid:
+        if not self._discard and self._assert and self.errors:
             yield from self.errors
-            for child in self.children.values():
+            for child in self.iter_children():
                 yield from child.collect_errors()
 
     def __str__(self) -> str:
