@@ -10,6 +10,13 @@ __all__ = [
 ]
 
 FormatValidator = Callable[[AnyJSONCompatible], None]
+"""The type of a ``"format"`` keyword validation function.
+
+A :class:`~jschon.vocabulary.format.FormatValidator` is a
+callable accepting a JSON-compatible Python object as its
+only argument. It must raise a :exc:`ValueError` if the
+argument is invalid per the applicable format specification.
+"""
 
 
 class FormatKeyword(Keyword):
@@ -18,9 +25,8 @@ class FormatKeyword(Keyword):
     def __init__(self, parentschema: JSONSchema, value: str):
         super().__init__(parentschema, value)
 
-        from jschon.catalogue import Catalogue
         try:
-            self.validator: FormatValidator = Catalogue.get_format_validator(value)
+            self.validator: FormatValidator = parentschema.catalogue.get_format_validator(value)
         except CatalogueError:
             self.validator = None
 
