@@ -1,4 +1,3 @@
-from jschon.catalogue import Catalogue
 from jschon.exceptions import JSONSchemaError
 from jschon.json import JSON
 from jschon.jsonschema import Keyword, JSONSchema, Scope, Applicator, ArrayApplicator
@@ -24,7 +23,10 @@ class RecursiveRefKeyword_2019_09(Keyword):
 
     def resolve(self) -> None:
         if (base_uri := self.parentschema.base_uri) is not None:
-            self.refschema = Catalogue.get_schema(base_uri, metaschema_uri=self.parentschema.metaschema_uri)
+            self.refschema = self.parentschema.catalogue.get_schema(
+                base_uri,
+                metaschema_uri=self.parentschema.metaschema_uri,
+            )
         else:
             raise JSONSchemaError(f'No base URI against which to resolve "$recursiveRef"')
 
@@ -32,15 +34,14 @@ class RecursiveRefKeyword_2019_09(Keyword):
         refschema = self.refschema
         if (recursive_anchor := refschema.get("$recursiveAnchor")) and \
                 recursive_anchor.value is True:
-            base_scope = scope.root
-            for key in scope.path:
-                if (base_schema := base_scope.schema) is refschema:
-                    break
-                if (base_anchor := base_schema.get("$recursiveAnchor")) and \
+
+            target_scope = scope
+            while target_scope is not None:
+                if (base_anchor := target_scope.schema.get("$recursiveAnchor")) and \
                         base_anchor.value is True:
-                    refschema = base_schema
-                    break
-                base_scope = base_scope.children[key]
+                    refschema = target_scope.schema
+
+                target_scope = target_scope.parent
 
         refschema.evaluate(instance, scope)
 
@@ -75,7 +76,7 @@ class ItemsKeyword_2019_09(Keyword, Applicator, ArrayApplicator):
             err_indices = []
             for index, item in enumerate(instance[:len(self.json)]):
                 eval_index = index
-                with scope(str(index)) as subscope:
+                with scope(item, str(index)) as subscope:
                     self.json[index].evaluate(item, subscope)
                     if not subscope.valid:
                         err_indices += [index]
@@ -92,7 +93,7 @@ class AdditionalItemsKeyword_2019_09(Keyword, Applicator):
     depends = "items"
 
     def evaluate(self, instance: JSON, scope: Scope) -> None:
-        if (items := scope.sibling("items")) and \
+        if (items := scope.sibling(instance, "items")) and \
                 (items_annotation := items.annotations.get("items")) and \
                 type(items_annotation.value) is int:
             annotation = None
